#ifndef _sudoku_h_
#define _sudoku_h_

// The coordinate system starts in the upper-left corner, at 0.
// For example, column 1 at the board level extends down through
// column 1 of squares[0][0], squares[1][0], and squares[2][0]

const int SQUARE_SIZE = 3;
const int SQUARE_INDEX_MAX = SQUARE_SIZE * SQUARE_SIZE;
const int SQUARE_VALUE_MAX = SQUARE_SIZE * SQUARE_SIZE;

// One of the subregions of the whole board
class SudokuSquare
{
public:
	SudokuSquare();

	// Returns true if the value is already present somewhere inside
	// the square
	bool is_present(int value) const;

	int square_data[SQUARE_SIZE][SQUARE_SIZE];
};

// The entire puzzle board
class SudokuBoard
{
public:
	// Sets the value of the cell at the specified column and row to
	// the specified value.  No checking on whether this is a valid
	// state or not is done.
	void set_value(int col, int row, int value);

	// Returns the value at the specified column and row.  Returns -1
	// if no value is set there.
	int get_value(int col, int row) const;

	// Check if it's valid to put the specified value at the specified
	// coordinates.  Note that this doesn't necessarily mean that it *is*
	// that value at that cell, just that it doesn't conflict with
	// any other known values at that time.
	bool is_valid(int col, int row, int value) const;

	// Check if the entire board is valid in its current state
	bool is_valid();

	// Check if the board is solved (filled with valid values)
	bool is_solved();

	// Get the first empty cell, checking column by column
	void next_empty(int& col, int& row);

	// Read the board in from the file
	void read(char *filename);
	// Print the current state of the board to stdout
	void print();

private:
	// Remap the global row and column coordinates into indexes into the squares
	// and square data arrays
	void remap_coords(int gcol, int grow, int& sqcol, int& sqrow, int& sqicol, int& sqirow) const;

	SudokuSquare squares[SQUARE_SIZE][SQUARE_SIZE];
};

// A function object for applying a particular strategy against the
// sudoku board.  Changes must be applied against the board passed
// in to the function operator.
class SudokuStrategy
{
public:
	// Returns true if it succeeded in adding to the board, false
	// if it failed to do so
	virtual bool operator()(SudokuBoard& board) = 0;
};

#endif
